TimerIconGroupWindow = class(Turbine.UI.Window)
function TimerIconGroupWindow:Constructor(uiData, parent)
    group_data = {}

    for key, value in pairs(uiData) do
        group_data[key] = value
    end

    self.width = group_data.Width or 100
    self.height = group_data.Height or 100
    self.x = 0--data.X
    self.y = 0--data.Y
    self.timerDirection = group_data.Timer_Stack_Direction
    self.sortOrder = group_data.Timer_Sorting

    Turbine.UI.Window.Constructor(self)
    self:SetParent(parent)
    self:SetSize(self.width, self.height)
    self:SetPosition(self.x, self.y)
    self:SetMouseVisible(false)
    self:SetVisible(true)
    --self:SetBackColor(Turbine.UI.Color.Red)

    self.itemID = 0

    self.labels = {}
    self.variableLUT = {}
    self.codeLUT = {}
    self.staticSortOrder = {}

    self.staticMode = group_data.Static

    self.IconListBox = Turbine.UI.ListBox()
    self.IconListBox:SetParent(parent)
    self.IconListBox:SetSize(self.width, self.height)
    self.IconListBox:SetMouseVisible(false)
    self.IconListBox:SetVisible(true)
    self.IconListBox:SetOrientation(Turbine.UI.Orientation[group_data.Orientation])
    self.IconListBox:SetFlippedLayout(group_data.Flipped_Layout)
    self.IconListBox:SetReverseFill(group_data.Reverse_Fill)
    self.IconListBox:SetMaxColumns(group_data.Max_Columns or 99)
    self.IconListBox:SetMaxRows(group_data.Max_Rows or 99)


    if self.staticMode and group_data.Use_Static_Order then
        self.sortMethod = function(a, b) 
            if a.Static_Sort_Order == b.Static_Sort_Order then
                return a.itemID < b.itemID
            else
                return a.Static_Sort_Order < b.Static_Sort_Order
            end
            
        end
    elseif self.sortOrder == "Alphabetical" then
        self.sortMethod = function(a, b) 
            if a:GetText() == b:GetText() then
                return a.itemID < b.itemID
            else
                return a:GetText() < b:GetText()
            end
            
        end
    elseif self.sortOrder == "Alphabetical Reverse" then
        self.sortMethod = function(a, b) 
            if a:GetText() == b:GetText() then
                return a.itemID < b.itemID
            else
                return a:GetText() > b:GetText()
            end
        end
    elseif self.sortOrder == "Low to High (Remaining duration)" then
        self.sortMethod = function(a, b) 
            if a:GetDuration() == b:GetDuration() then
                return a.itemID < b.itemID
            else
                return a:GetDuration() < b:GetDuration()
            end
        end
    elseif self.sortOrder == "High to Low (Max duration)" then
        self.sortMethod = function(a, b) 
            if a:GetMaxDuration() == b:GetMaxDuration() then
                return a.itemID < b.itemID
            else
                return a:GetMaxDuration() > b:GetMaxDuration()
            end
        end
    elseif self.sortOrder == "Low to High (Max duration)" then
        self.sortMethod = function(a, b) 
            if a:GetMaxDuration() == b:GetMaxDuration() then
                return a.itemID < b.itemID
            else
                return a:GetMaxDuration() < b:GetMaxDuration()
            end
        end
    else -- High to Low (Remaining duration)
        self.sortMethod = function(a, b) 
            if a:GetDuration() == b:GetDuration() then
                return a.itemID < b.itemID
            else
                return a:GetDuration() > b:GetDuration()
            end
        end
    end

end

function TimerIconGroupWindow:RemoveLabel(itemID)
    if not self.staticMode and self.labels[itemID] then
        self.IconListBox:RemoveItem(self.labels[itemID])
        self.labels[itemID]:SetVisible(false)
        self.labels[itemID]:Unload()
        self.labels[itemID] = nil
    end

end

function TimerIconGroupWindow:StartIcon(code, layoutTable, staticCreation)    
    if self.codeLUT[code] then
        if self.labels[self.codeLUT[code]] and self.IconListBox:ContainsItem(self.labels[self.codeLUT[code]]) then
            self.labels[self.codeLUT[code]]:Trigger({["Timer"] = true, ["Timer_Duration"] = layoutTable.Time or 1}, {["Timer_Duration"] = layoutTable.Time or 1})
            self.IconListBox:Sort(self.sortMethod)
            return
        end
    end

    local newID = self.itemID
    layoutTable.Width = layoutTable.Icon_Width or 32
    layoutTable.Height = layoutTable.Icon_Height or 32
    local valuebar = TimerLabel(self, 0, 0, layoutTable, function() 
        self:RemoveLabel(newID) 
    end)

    valuebar.itemID = newID
    valuebar.Static_Sort_Order = layoutTable.Sort_Order or 0
    self.labels[self.itemID] = valuebar
    if layoutTable.Variable_Name ~= "" then
        if not self.variableLUT[layoutTable.Variable_Name] then
            self.variableLUT[layoutTable.Variable_Name] = {}
        end
        for key, value in pairs(self.variableLUT[layoutTable.Variable_Name]) do
            if self.labels[key] and self.IconListBox:ContainsItem(self.labels[key]) then
                self.labels[key]:Trigger({["Timer"] = true, ["Timer_Duration"] = 0}, {["Timer_Duration"] = 0})
                self.labels[key]:SetVisible(false)
                self:RemoveLabel(key)
            end
        end
        self.variableLUT[layoutTable.Variable_Name][self.itemID] = true
    end
    self.codeLUT[code] = self.itemID

    if not staticCreation then
        valuebar:Trigger({["Timer"] = true, ["Timer_Duration"] = layoutTable.Time or 1}, {["Timer_Duration"] = layoutTable.Time or 1})
    end
    if staticCreation and self.variableLUT[layoutTable.Variable_Name] and self.variableLUT[layoutTable.Variable_Name][self.itemID] then
        local alreadyExisting = false
        for key, value in pairs(self.variableLUT[layoutTable.Variable_Name]) do
            if self.labels[key] then
                if self.IconListBox:ContainsItem(self.labels[key]) then
                    
                    alreadyExisting = true
                end
            end
        end

        if not alreadyExisting then
            self.IconListBox:AddItem(valuebar)
            self.IconListBox:Sort(self.sortMethod)
        end
    else
        self.IconListBox:AddItem(valuebar)
        self.IconListBox:Sort(self.sortMethod)
    end

    self.itemID = self.itemID + 1
    
end

function TimerIconGroupWindow:StartStaticIcon(code, layoutTable)
    if self.codeLUT[code] and self.labels[self.codeLUT[code]] then
        local id = self.codeLUT[code]
        if self.variableLUT[layoutTable.Variable_Name] and self.variableLUT[layoutTable.Variable_Name][id] then
            if not self.IconListBox:ContainsItem(self.labels[id]) then
                for key, value in pairs(self.variableLUT[layoutTable.Variable_Name]) do
                    if self.labels[key] and self.IconListBox:ContainsItem(self.labels[key]) then
                        local index = self.IconListBox:IndexOfItem(self.labels[key])
                        self.IconListBox:RemoveItemAt(index)
                        self.labels[key]:Trigger({["Timer"] = true, ["Timer_Duration"] = 0}, {["Timer_Duration"] = 0})
                        self.labels[key]:SetVisible(false)
                        self.IconListBox:AddItem(self.labels[id])
                        self.IconListBox:SetItem(index, self.labels[id])
                        break
                    end
                end
            end
        end
        self.labels[self.codeLUT[code]]:Trigger({["Timer"] = true, ["Timer_Duration"] = layoutTable.Time or 1}, {["Timer_Duration"] = layoutTable.Time or 1})
    end
end

function TimerIconGroupWindow:Unload()
    for i = 1, self.IconListBox:GetItemCount() do
        self.IconListBox:GetItem(i):SetVisible(false)
    end
    self.IconListBox:ClearItems()
    self.labels = nil
    self.variableLUT = nil
    self.codeLUT = nil
end